"""Test cases for static IPv4 routes."""

import json

from pyangbind.lib import pybindJSON
from retry import retry

from oc_config_validate import schema, testbase
from oc_config_validate.models.local_routing import openconfig_local_routing


class AddStaticRoute(testbase.TestCase):
    """Tests configuring a static route.

    1. A gNMI Set message is sent to configure the route.
    1. A gNMI Get message on the /config and /state containers validates it.

    All arguments are read from the Test YAML description.

    Args:
        description: Optional text description of the route.
        prefix: Destination prefix of the static route.
        next_hop: IP of the next hop of the route.
        index: Index of the next hop for the prefix. Defaults to 0.
        metric: Optional numeric metric of the next hop for the prefix.
    """
    description = None
    prefix = ""
    next_hop = ""
    index = 0
    metric = None
    route = None

    def setUp(self):
        self.route = openconfig_local_routing(
        ).local_routes.static_routes.static.add(self.prefix)
        self.route.config.prefix = self.prefix
        if self.description is not None:
            self.route.config.description = self.description
        self.nh = self.route.next_hops.next_hop.add(self.index)
        self.nh.config.index = self.index
        self.nh.config.next_hop = self.next_hop
        if self.metric is not None:
            self.nh.config.metric = self.metric

    def test0001(self):
        self.assertArgs(["prefix", "next_hop"])

    def test0100(self):
        xpath = "/local-routes/static-routes/static[prefix=%s]" % self.prefix
        _json_value = json.loads(pybindJSON.dumps(self.route, mode='ietf'))
        self.assertTrue(self.gNMISetUpdate(xpath, _json_value),
                        "gNMI Set did not succeed.")

    @retry(exceptions=AssertionError, tries=3, delay=10)
    def test0200(self):
        xpath = ("/local-routes/static-routes/static[prefix=%s]/"
                 "next-hops/next-hop[index=%d]/config") % (
            self.prefix, self.index)
        resp_val = self.gNMIGetAssertJson(xpath)
        self.assertJsonModel(
            resp_val, self.nh.config,
            'gNMI Get on the /config container does not match model')
        want = json.loads(
            schema.removeOpenConfigPrefix(
                pybindJSON.dumps(self.nh.config, mode='ietf')))
        self.assertJsonCmp(resp_val, want)

    @retry(exceptions=AssertionError, tries=3, delay=10)
    def test0300(self):
        xpath = ("/local-routes/static-routes/static[prefix=%s]/"
                 "next-hops/next-hop[index=%d]/state") % (
            self.prefix, self.index)
        resp_val = self.gNMIGetAssertJson(xpath)
        self.nh.state._set_index(self.index)
        self.nh.state._set_next_hop(self.next_hop)
        if self.metric is not None:
            self.nh.state._set_metric(self.metric)
        self.assertJsonModel(
            resp_val, self.nh.state,
            'gNMI Get on the /state container does not match model')
        want = json.loads(
            schema.removeOpenConfigPrefix(
                pybindJSON.dumps(self.nh.state, mode='ietf')))
        self.assertJsonCmp(resp_val, want)


class RemoveStaticRoute(testbase.TestCase):
    """Tests removing a static route.

    1. gNMI Get message on the /config container, to check it is configured.
    1. gNMI Set message to delete the route.
    1. gNMI Get message on the /config container to check it is not there.

    All arguments are read from the Test YAML description.

    Args:
        prefix: Destination prefix of the static route.
        index: Index of the next hop for the prefix. Defaults to 0.
    """
    prefix = ""
    index = 0

    def test0000(self):
        self.assertArgs(["prefix"])

    def test0100(self):
        xpath = ("/local-routes/static-routes/static[prefix=%s]"
                 "/next-hops/next-hop[index=%d]") % (
            self.prefix, self.index)
        if not self.gNMIGet(xpath + "/config"):
            self.log("Route to %s via next-hop %d not configured",
                     self.prefix, self.index)
            return
        self.assertTrue(self.gNMISetDelete(xpath),
                        "gNMI Delete did not succeed.")

    @retry(exceptions=AssertionError, tries=3, delay=10)
    def test0200(self):
        xpath = ("/local-routes/static-routes/static[prefix=%s]"
                 "/next-hops/next-hop[index=%d]/config") % (
            self.prefix, self.index)
        resp = self.gNMIGet(xpath)
        self.assertIsNone(
            resp,
            "Route to %s via next-hop %d still configured" % (self.prefix,
                                                              self.index))

    @retry(exceptions=AssertionError, tries=3, delay=10)
    def test0300(self):
        xpath = ("/local-routes/static-routes/static[prefix=%s]"
                 "/next-hops/next-hop[index=%d]/state") % (
            self.prefix, self.index)
        resp = self.gNMIGet(xpath)
        self.assertIsNone(
            resp,
            "Route to %s via next-hop %d still present" % (self.prefix,
                                                           self.index))


class CheckRouteState(testbase.TestCase):
    """Checks the state on a static route.

    1. A gNMI Get message on the /state container.

    All arguments are read from the Test YAML description.

    Args:
        description: Optional text description of the route.
        prefix: Destination prefix of the static route.
        next_hop: IP of the next hop of the route.
        index: Index of the next hop for the prefix. Defaults to 0.
        metric: Optional numeric metric of the next hop for the prefix.
    """
    description = None
    prefix = ""
    next_hop = ""
    index = 0
    metric = None

    @retry(exceptions=AssertionError, tries=3, delay=10)
    def test0100(self):
        """"""
        xpath = ("/local-routes/static-routes/static[prefix=%s]"
                 "/next-hops/next-hop[index=%d]/state") % (
            self.prefix, self.index)

        ste = openconfig_local_routing().local_routes.static_routes.static.\
            add(prefix=self.prefix).next_hops.next_hop.add(index=self.index).\
            state
        ste._set_index(self.index)
        ste._set_next_hop(self.next_hop)
        if self.metric is not None:
            ste._set_metric(self.metric)
        resp_val = self.gNMIGetAssertJson(xpath)
        self.assertJsonModel(
            resp_val, ste,
            'gNMI Get on the /state container does not match model')
        want = json.loads(
            schema.removeOpenConfigPrefix(
                pybindJSON.dumps(ste, mode='ietf')))
        self.assertJsonCmp(resp_val, want)


class CheckRouteConfig(testbase.TestCase):
    """Checks the configuration on a static route.

    1. A gNMI Get message on the /config container.

    All arguments are read from the Test YAML description.

    Args:
        description: Optional text description of the route.
        prefix: Destination prefix of the static route.
        next_hop: IP of the next hop of the route.
        index: Index of the next hop for the prefix. Defaults to 0.
        metric: Optional numeric metric of the next hop for the prefix.
    """
    description = None
    prefix = ""
    next_hop = ""
    index = 0
    metric = None

    @retry(exceptions=AssertionError, tries=3, delay=10)
    def test0100(self):
        """"""
        xpath = ("/local-routes/static-routes/static[prefix=%s]"
                 "/next-hops/next-hop[index=%d]/config") % (
            self.prefix, self.index)

        cfg = openconfig_local_routing().local_routes.static_routes.static.\
            add(prefix=self.prefix).next_hops.next_hop.add(index=self.index).\
            config.config()
        cfg._set_index(self.index)
        cfg._set_next_hop(self.next_hop)
        if self.metric is not None:
            cfg._set_metric(self.metric)
        resp_val = self.gNMIGetAssertJson(xpath)
        self.assertJsonModel(
            resp_val, cfg,
            'gNMI Get on the /config container does not match model')
        want = json.loads(
            schema.removeOpenConfigPrefix(
                pybindJSON.dumps(cfg, mode='ietf')))
        self.assertJsonCmp(resp_val, want)



# -*- coding: utf-8 -*-
from operator import attrgetter
from pyangbind.lib.yangtypes import RestrictedPrecisionDecimalType
from pyangbind.lib.yangtypes import RestrictedClassType
from pyangbind.lib.yangtypes import TypedListType
from pyangbind.lib.yangtypes import YANGBool
from pyangbind.lib.yangtypes import YANGListType
from pyangbind.lib.yangtypes import YANGDynClass
from pyangbind.lib.yangtypes import ReferenceType
from pyangbind.lib.base import PybindBase
from collections import OrderedDict
from decimal import Decimal
from bitarray import bitarray
import six

# PY3 support of some PY2 keywords (needs improved)
if six.PY3:
  import builtins as __builtin__
  long = int
elif six.PY2:
  import __builtin__

class openconfig_isis_types(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-isis-types - based on the path /openconfig-isis-types. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module contains general data definitions for use in ISIS YANG
model.
  """
  _pyangbind_elements = {}

  

class openconfig_isis_routing(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-isis-routing - based on the path /openconfig-isis-routing. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module describes YANG model for ISIS Routing
  """
  _pyangbind_elements = {}

  

class openconfig_isis_policy(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-isis-policy - based on the path /openconfig-isis-policy. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module contains data definitions for ISIS routing policy.
It augments the base routing-policy module with BGP-specific
options for conditions and actions.
  """
  _pyangbind_elements = {}

  

class openconfig_isis_lsp(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-isis-lsp - based on the path /openconfig-isis-lsp. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This sub-module describes a YANG model for the IS-IS Link State
Database (LSDB).

Portions of this code were derived from IETF RFCs relating to the
IS-IS protocol.
Please reproduce this note if possible.
IETF code is subject to the following copyright and license:
Copyright (c) IETF Trust and the persons identified as authors of
the code.
All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, is permitted pursuant to, and subject to the license
terms contained in, the Simplified BSD License set forth in
Section 4.c of the IETF Trust's Legal Provisions Relating
to IETF Documents (http://trustee.ietf.org/license-info).
  """
  _pyangbind_elements = {}

  

class openconfig_isis_lsdb_types(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-isis-lsdb-types - based on the path /openconfig-isis-lsdb-types. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module contains general LSDB type definitions for use in
ISIS YANG model. 
  """
  _pyangbind_elements = {}

  

class openconfig_isis(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-isis - based on the path /openconfig-isis. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module describes a YANG model for ISIS protocol configuration.
It is a limited subset of all of the configuration parameters
available in the variety of vendor implementations, hence it is
expected that it would be augmented with vendor - specific configuration
data as needed. Additional modules or submodules to handle other
aspects of ISIS configuration, including policy, routing, types,
LSDB and additional address families are also expected. This model
supports the following ISIS configuration level hierarchy:

ISIS
+-> { global ISIS configuration}
   +-> levels +-> { level config}
       +-> { system-level-counters }
       +-> { level link-state-database}
   +-> interface +-> { interface config }
       +-> { circuit-counters }
       +-> { levels config }
       +-> { level adjacencies }
  """
  _pyangbind_elements = {}

  

class openconfig_isis_lsp(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-isis-lsp - based on the path /openconfig-isis-lsp. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This sub-module describes a YANG model for the IS-IS Link State
Database (LSDB).

Portions of this code were derived from IETF RFCs relating to the
IS-IS protocol.
Please reproduce this note if possible.
IETF code is subject to the following copyright and license:
Copyright (c) IETF Trust and the persons identified as authors of
the code.
All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, is permitted pursuant to, and subject to the license
terms contained in, the Simplified BSD License set forth in
Section 4.c of the IETF Trust's Legal Provisions Relating
to IETF Documents (http://trustee.ietf.org/license-info).
  """
  _pyangbind_elements = {}

  

class openconfig_isis_routing(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-isis-routing - based on the path /openconfig-isis-routing. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module describes YANG model for ISIS Routing
  """
  _pyangbind_elements = {}

  




# -*- coding: utf-8 -*-
from operator import attrgetter
from pyangbind.lib.yangtypes import RestrictedPrecisionDecimalType
from pyangbind.lib.yangtypes import RestrictedClassType
from pyangbind.lib.yangtypes import TypedListType
from pyangbind.lib.yangtypes import YANGBool
from pyangbind.lib.yangtypes import YANGListType
from pyangbind.lib.yangtypes import YANGDynClass
from pyangbind.lib.yangtypes import ReferenceType
from pyangbind.lib.base import PybindBase
from collections import OrderedDict
from decimal import Decimal
from bitarray import bitarray
import six

# PY3 support of some PY2 keywords (needs improved)
if six.PY3:
  import builtins as __builtin__
  long = int
elif six.PY2:
  import __builtin__

class openconfig_local_routing(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-local-routing - based on the path /openconfig-local-routing. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module describes configuration and operational state data
for routes that are locally generated, i.e., not created by
dynamic routing protocols.  These include static routes, locally
created aggregate routes for reducing the number of constituent
routes that must be advertised, summary routes for IGPs, etc.

This model expresses locally generated routes as generically as
possible, avoiding configuration of protocol-specific attributes
at the time of route creation.  This is primarily to avoid
assumptions about how underlying router implementations handle
route attributes in various routing table data structures they
maintain.  Hence, the definition of locally generated routes
essentially creates 'bare' routes that do not have any protocol-
specific attributes.

When protocol-specific attributes must be attached to a route
(e.g., communities on a locally defined route meant to be
advertised via BGP), the attributes should be attached via a
protocol-specific policy after importing the route into the
protocol for distribution (again via routing policy).
  """
  _pyangbind_elements = {}

  




# -*- coding: utf-8 -*-
from operator import attrgetter
from pyangbind.lib.yangtypes import RestrictedPrecisionDecimalType
from pyangbind.lib.yangtypes import RestrictedClassType
from pyangbind.lib.yangtypes import TypedListType
from pyangbind.lib.yangtypes import YANGBool
from pyangbind.lib.yangtypes import YANGListType
from pyangbind.lib.yangtypes import YANGDynClass
from pyangbind.lib.yangtypes import ReferenceType
from pyangbind.lib.base import PybindBase
from collections import OrderedDict
from decimal import Decimal
from bitarray import bitarray
import six

# PY3 support of some PY2 keywords (needs improved)
if six.PY3:
  import builtins as __builtin__
  long = int
elif six.PY2:
  import __builtin__

class openconfig_vlan(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-vlan - based on the path /openconfig-vlan. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module defines configuration and state variables for VLANs,
in addition to VLAN parameters associated with interfaces
  """
  _pyangbind_elements = {}

  

class openconfig_vlan_types(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-vlan-types - based on the path /openconfig-vlan-types. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module defines configuration and state variables for VLANs,
in addition to VLAN parameters associated with interfaces
  """
  _pyangbind_elements = {}

  




# -*- coding: utf-8 -*-
from operator import attrgetter
from pyangbind.lib.yangtypes import RestrictedPrecisionDecimalType
from pyangbind.lib.yangtypes import RestrictedClassType
from pyangbind.lib.yangtypes import TypedListType
from pyangbind.lib.yangtypes import YANGBool
from pyangbind.lib.yangtypes import YANGListType
from pyangbind.lib.yangtypes import YANGDynClass
from pyangbind.lib.yangtypes import ReferenceType
from pyangbind.lib.base import PybindBase
from collections import OrderedDict
from decimal import Decimal
from bitarray import bitarray
import six

# PY3 support of some PY2 keywords (needs improved)
if six.PY3:
  import builtins as __builtin__
  long = int
elif six.PY2:
  import __builtin__

class openconfig_p4rt(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-p4rt - based on the path /openconfig-p4rt. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module defines a set of extensions that provide P4Runtime (P4RT)
specific extensions to the OpenConfig data models. Specifically, these
parameters for configuration and state provide extensions that control
the P4RT service, or allow it to be used alongside other OpenConfig
data models.

The P4RT protocol specification is linked from https://p4.org/specs/
under the P4Runtime heading.
  """
  _pyangbind_elements = {}

  




"""Copyright 2021 Google LLC.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.

You may obtain a copy of the License at
            https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""
import argparse
import logging
import os
import sys
import time
from typing import Any, Dict

import yaml

from oc_config_validate import (__version__, context, formatter, runner,
                                schema, target, testbase)

LOGGING_FORMAT = "%(levelname)s(%(filename)s:%(lineno)d):%(message)s"


def createArgsParser() -> argparse.ArgumentParser:
    """Create parser for arguments passed into the program from the CLI.

     Returns:
        argparse.ArgumentParser object.

     """
    parser = argparse.ArgumentParser(
        description="OpenConfig Configuration Validation utility.")
    parser.add_argument(
        "-tgt",
        "--target",
        type=str,
        help="The gNMI Target, as hostname:port.",
    )
    parser.add_argument(
        "-user",
        "--username",
        type=str,
        help="Username to use when establishing a gNMI Channel to the Target.",
    )
    parser.add_argument(
        "-pass",
        "--password",
        type=str,
        help="Password to use when establishing a gNMI Channel to the Target.",
    )
    parser.add_argument(
        "-key",
        "--private_key",
        type=str,
        help="Path to the Private key to use when establishing"
        "a gNMI Channel to the Target.",
    )
    parser.add_argument(
        "-ca",
        "--root_ca_cert",
        type=str,
        help="Path to Root CA to use when building the gNMI Channel.",
    )
    parser.add_argument(
        "-cert",
        "--cert_chain",
        type=str,
        help="Path to Certificate chain to use when"
        "establishing a gNMI Channel to the Target.")
    parser.add_argument(
        "-tests",
        "--tests_file",
        type=str,
        action="store",
        help="YAML file to read the test to run.")
    parser.add_argument(
        "-init",
        "--init_config_file",
        type=str,
        action="store",
        help="JSON file with the initial full OpenConfig configuration to "
        "apply.")
    parser.add_argument(
        "-xpath",
        "--init_config_xpath",
        type=str,
        action="store",
        help="gNMI xpath where to apply the initial config.",
        default="/")
    parser.add_argument(
        "-results",
        "--results_file",
        type=str,
        action="store",
        help="Filename where to write the test results.")
    parser.add_argument(
        "-f",
        "--format",
        type=str,
        action="store",
        help="Format "
        "of the GetResponse to be printed. Default=JSON.",
        choices=["json", "protobuff"],
        default="json")
    parser.add_argument(
        "-v", "--version", help="Print program version", action="store_true")
    parser.add_argument(
        "-V", "--verbose", help="Enable gRPC debugging and extra logging.",
        action="store_true")
    parser.add_argument(
        "-models", "--oc_models_versions", help="Print OC models versions.",
        action="store_true")
    parser.add_argument(
        "--no_tls", help="gRPC insecure mode.", action="store_true")
    parser.add_argument(
        "-o",
        "--tls_host_override",
        type=str,
        action="store",
        help="Hostname to use during the TLS certificate check.",
    )
    parser.add_argument(
        "--target_cert_as_root_ca",
        action="store_true",
        help="Fetch the Target TLS cert and use it as client Root CA cert.",
    )
    parser.add_argument(
        "-set_cooldown",
        "--gnmi_set_cooldown_secs",
        type=int,
        action="store",
        help="Seconds to wait after a successful gNMI Set message.",
    )
    parser.add_argument(
        "--stop_on_error",
        action="store_true",
        help="Stop the execution if a test fails.",
    )
    parser.add_argument(
        "--log_gnmi",
        action="store_true",
        help="Log the gnmi requests to the tests results.",
    )
    return parser


def validateArgs(args: Dict[str, Any]):
    """Returns True if the arguments are valid.

     Raises:
       ValueError if any argument is invalid.
       IOError is unable to open a file given in argument.

    """

    def isFileOK(filename: str, writable: bool = False):
        try:
            file = open(filename, "w+" if writable else "r", encoding="utf8")
            file.close()
        except IOError as io_error:
            logging.error("Unable to open %s: %s", filename, io_error)
            raise

    # Mandatory args for tests
    for arg, write in [("tests_file", False), ("results_file", True)]:
        if not args[arg]:
            raise ValueError("Needed --%s file" % arg)
        isFileOK(args[arg], write)

    if args["init_config_file"]:
        isFileOK(args["init_config_file"], False)

    # Output format supported
    if (args["format"] and
            args["format"].lower() not in formatter.SUPPORTED_FORMATS):
        raise ValueError("Output format %s is not supported.")


def main():  # noqa
    """Executes this library."""
    argparser = createArgsParser()
    args = vars(argparser.parse_args())

    if args["version"]:
        print(__version__)
        sys.exit()
    if args["oc_models_versions"]:
        print(schema.getOcModelsVersions())
        sys.exit()

    if args["verbose"]:
        # os.environ["GRPC_TRACE"] = "all"
        os.environ["GRPC_VERBOSITY"] = "DEBUG"
    logging.basicConfig(
        level=logging.DEBUG if args["verbose"] else logging.INFO,
        format=LOGGING_FORMAT)

    try:
        validateArgs(args)
    except (IOError, ValueError) as error:
        sys.exit("Invalid arguments: %s" % error)

    if args["log_gnmi"]:
        testbase.LOG_GNMI = args["log_gnmi"]

    try:
        ctx = context.fromFile(args["tests_file"])
    except IOError as io_error:
        sys.exit("Unable to read %s: %s" % (args["tests_file"], io_error))
    except yaml.YAMLError as yaml_error:
        sys.exit("Unable to parse YAML file %s: %s" % (args["tests_file"],
                 yaml_error))

    logging.info("Read tests file '%s': %d tests to run",
                 args["tests_file"], len(ctx.tests))

    # Override Target options
    for arg in ["target", "username", "password", "no_tls", "private_key",
                "cert_chain", "root_ca_cert", "tls_host_override",
                "target_cert_as_root_ca", "gnmi_set_cooldown_secs"]:
        if args[arg]:
            setattr(ctx.target, arg, args[arg])
    try:
        ctx.target.validate()
    except ValueError as error:
        sys.exit("Invalid Target: %s" % error)

    # Concatenate initial configuration paths and files
    if args["init_config_file"]:
        ctx.init_configs.append(context.InitConfig(args["init_config_file"],
                                                   args["init_config_xpath"]))

    with target.TestTarget(ctx.target) as tgt:
        try:
            runner.setInitConfigs(ctx, tgt,
                                  stop_on_error=args["stop_on_error"])
        except runner.InitConfigError as err:
            sys.exit("Unable to apply init config(s): %s" % err)

        start_t = time.time()
        results = runner.runTests(
            ctx, tgt, stop_on_error=args["stop_on_error"])
        end_t = time.time()

    test_run = testbase.TestRun(ctx, tgt)
    test_run.copyResults(results, start_t, end_t)
    logging.info("Results Summary: %s", test_run.summary())

    try:
        fmtr = formatter.makeFormatter(args["format"])
        fmtr.writeResultsToFile(test_run, args["results_file"])
        logging.info("Test results written to %s", args["results_file"])
    except IOError as io_error:
        logging.exception("Unable to write file %s: %s", args["results_file"],
                          io_error)
    except TypeError as type_error:
        logging.exception("Unable to parse results into a JSON text: %s",
                          type_error)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from operator import attrgetter
from pyangbind.lib.yangtypes import RestrictedPrecisionDecimalType
from pyangbind.lib.yangtypes import RestrictedClassType
from pyangbind.lib.yangtypes import TypedListType
from pyangbind.lib.yangtypes import YANGBool
from pyangbind.lib.yangtypes import YANGListType
from pyangbind.lib.yangtypes import YANGDynClass
from pyangbind.lib.yangtypes import ReferenceType
from pyangbind.lib.base import PybindBase
from collections import OrderedDict
from decimal import Decimal
from bitarray import bitarray
import six

# PY3 support of some PY2 keywords (needs improved)
if six.PY3:
  import builtins as __builtin__
  long = int
elif six.PY2:
  import __builtin__

class openconfig_aft_types(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-types - based on the path /openconfig-aft-types. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Types related to the OpenConfig Abstract Forwarding
Table (AFT) model
  """
  _pyangbind_elements = {}

  

class openconfig_aft_pf(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-pf - based on the path /openconfig-aft-pf. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the abstract
forwarding table(s) for policy forwarding entries. These are
defined to be forwarding tables that allow matches on
fields other than the destination address that is used in
other forwarding tables.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-common - based on the path /openconfig-aft-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings that are re-used
across multiple contexts within the AFT model.
  """
  _pyangbind_elements = {}

  

class openconfig_aft(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft - based on the path /openconfig-aft. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: A model describing the forwarding entries installed on a network
element. It should be noted that this model is not expected to
align 1:1 with the underlying structure used directly by a
forwarding element (e.g., linecard), but rather provide an
abstraction that can be consumed by an NMS to observe, and in some
cases manipulate, the internal forwarding database in a simplified
manner. Since the underlying model of the forwarding table is not
expected to align with this model, the structure described herein
is referred to as an Abstract Forwarding Table (AFT), rather than
the FIB.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_ipv4(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-ipv4 - based on the path /openconfig-aft-ipv4. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the abstract
forwarding tables for IPv4.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_ipv6(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-ipv6 - based on the path /openconfig-aft-ipv6. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the abstract
forwarding tables for IPv6.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_mpls(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-mpls - based on the path /openconfig-aft-mpls. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the abstract
forwarding table for MPLS label forwarding.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_pf(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-pf - based on the path /openconfig-aft-pf. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the abstract
forwarding table(s) for policy forwarding entries. These are
defined to be forwarding tables that allow matches on
fields other than the destination address that is used in
other forwarding tables.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_ethernet(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-ethernet - based on the path /openconfig-aft-ethernet. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the abstract
forwarding tables for Ethernet.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-common - based on the path /openconfig-aft-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings that are re-used
across multiple contexts within the AFT model.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_state_synced(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-state-synced - based on the path /openconfig-aft-state-synced. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the state
synced signals corresponding to various abstract forwarding tables.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_ipv6(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-ipv6 - based on the path /openconfig-aft-ipv6. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the abstract
forwarding tables for IPv6.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-common - based on the path /openconfig-aft-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings that are re-used
across multiple contexts within the AFT model.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-common - based on the path /openconfig-aft-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings that are re-used
across multiple contexts within the AFT model.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_ethernet(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-ethernet - based on the path /openconfig-aft-ethernet. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the abstract
forwarding tables for Ethernet.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-common - based on the path /openconfig-aft-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings that are re-used
across multiple contexts within the AFT model.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_ipv4(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-ipv4 - based on the path /openconfig-aft-ipv4. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the abstract
forwarding tables for IPv4.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-common - based on the path /openconfig-aft-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings that are re-used
across multiple contexts within the AFT model.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_mpls(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-mpls - based on the path /openconfig-aft-mpls. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the abstract
forwarding table for MPLS label forwarding.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_common(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-common - based on the path /openconfig-aft-common. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings that are re-used
across multiple contexts within the AFT model.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_state_synced(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-state-synced - based on the path /openconfig-aft-state-synced. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: Submodule containing definitions of groupings for the state
synced signals corresponding to various abstract forwarding tables.
  """
  _pyangbind_elements = {}

  

class openconfig_aft_network_instance(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module openconfig-aft-network-instance - based on the path /openconfig-aft-network-instance. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.

  YANG Description: This module provides augmentations that are utilized
when building the OpenConfig network instance model to
add per-NI AFTs.
  """
  _pyangbind_elements = {}

  

